/***************************************************
 * This is our touchscreen painting example for the Adafruit ILI9341 Shield
 * ----> http://www.adafruit.com/products/1651
 * 
 * Check out the links above for our tutorials and wiring diagrams
 * These displays use SPI to communicate, 4 or 5 pins are required to
 * interface (RST is optional)
 * Adafruit invests time and resources providing this open source code,
 * please support Adafruit and open-source hardware by purchasing
 * products from Adafruit!
 * 
 * Written by Limor Fried/Ladyada for Adafruit Industries.
 * MIT license, all text above must be included in any redistribution
 ****************************************************/


#include <Adafruit_GFX.h>    // Core graphics library
#include <SPI.h>
#include <Wire.h>      // this is needed even tho we aren't using it
#include <Optimized_ILI9341.h>
#include <Adafruit_STMPE610.h>

// This is calibration data for the raw touch data to the screen coordinates
#define TS_MINX 150
#define TS_MINY 130
#define TS_MAXX 3800
#define TS_MAXY 4000

// The STMPE610 uses hardware SPI on the shield, and #8
#define STMPE_CS 8
Adafruit_STMPE610 ts = Adafruit_STMPE610(STMPE_CS);

// The display also uses hardware SPI, plus #9 & #10
#define TFT_CS 10
#define TFT_DC  9
#define TFT_RST -1
Optimized_ILI9341 tft = Optimized_ILI9341(TFT_CS, TFT_DC);

// Size of the color selection boxes and the paintbrush size
#define BOXSIZE 40
#define PENRADIUS 3
int oldcolor, currentcolor;

void PrintPinsConfig(boolean fCTar) {
  Serial.print(CORE_PIN6_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN8_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN9_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN10_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN11_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN12_CONFIG, HEX);
  Serial.print(" ");
  if (fCTar) {
    Serial.print(CORE_PIN13_CONFIG, HEX);
    Serial.print(" ");
    uint32_t t = SPI0_CTAR0;
    Serial.print(t, HEX);
    Serial.print(" ");
    t = SPI0_CTAR1;
    Serial.print(t, HEX);
    Serial.print(" ");
    t = SPI0_MCR;
    Serial.print(t, HEX);
  }
  Serial.println();
}



void setup(void) {
  // while (!Serial);     // used for leonardo debugging
  pinMode(2, OUTPUT);
  digitalWrite(2, LOW);
  delay(10);
  digitalWrite(2, HIGH);
  delay(10);
  digitalWrite(2, LOW);
  delay(2000);
  Serial.begin(9600);
  Serial.println(F("Touch Paint!"));

  PrintPinsConfig(false);


  tft.begin();
  PrintPinsConfig(true);

  // hack wait for all outputs to complete clear out any pending reads...
  // clear out any current received bytes
  uint16_t wTimeout = 0xffff;
  uint8_t r;
  while (((SPI0.SR) & (15 << 12)) && (--wTimeout)) ; // wait until empty
  wTimeout = 0x10;    // should not go more than 4...
  while ((((SPI0.SR) >> 4) & 0xf) && (--wTimeout))  {
    r = SPI0.POPR;
  }


  digitalWrite(2, HIGH);
  if (!ts.begin()) {
    digitalWrite(2, LOW);
    Serial.println("Couldn't start touchscreen controller");
    PrintPinsConfig(true);
    while (1);
  }
  digitalWrite(2, LOW);

  PrintPinsConfig(true);
  Serial.println("Touchscreen started");

  tft.fillScreen(ILI9341_BLACK);

  // make the color selection boxes
  tft.fillRect(0, 0, BOXSIZE, BOXSIZE, ILI9341_RED);
  tft.fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, ILI9341_YELLOW);
  tft.fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, ILI9341_GREEN);
  tft.fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, ILI9341_CYAN);
  tft.fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, ILI9341_BLUE);
  tft.fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, ILI9341_MAGENTA);

  // select the current color 'red'
  tft.drawRect(0, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
  currentcolor = ILI9341_RED;
}


void loop()
{
  // See if there's any  touch data for us
  digitalWrite(2, HIGH);
  if (ts.bufferEmpty()) {
    digitalWrite(2, LOW);
    return;
  }
  /*
  // You can also wait for a touch
   if (! ts.touched()) {
   return;
   }
   */

  // Retrieve a point  
  TS_Point p = ts.getPoint();
  digitalWrite(2, LOW);

  /*
  Serial.print("X = "); Serial.print(p.x);
   Serial.print("\tY = "); Serial.print(p.y);
   Serial.print("\tPressure = "); Serial.println(p.z);  
   */

  // Scale from ~0->4000 to tft.width using the calibration #'s
  p.x = map(p.x, TS_MINX, TS_MAXX, 0, tft.width());
  p.y = map(p.y, TS_MINY, TS_MAXY, 0, tft.height());

  /*
  Serial.print("("); Serial.print(p.x);
   Serial.print(", "); Serial.print(p.y);
   Serial.println(")");
   */

  if (p.y < BOXSIZE) {
    oldcolor = currentcolor;

    if (p.x < BOXSIZE) { 
      currentcolor = ILI9341_RED; 
      tft.drawRect(0, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    } 
    else if (p.x < BOXSIZE*2) {
      currentcolor = ILI9341_YELLOW;
      tft.drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    } 
    else if (p.x < BOXSIZE*3) {
      currentcolor = ILI9341_GREEN;
      tft.drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    } 
    else if (p.x < BOXSIZE*4) {
      currentcolor = ILI9341_CYAN;
      tft.drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    } 
    else if (p.x < BOXSIZE*5) {
      currentcolor = ILI9341_BLUE;
      tft.drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    } 
    else if (p.x < BOXSIZE*6) {
      currentcolor = ILI9341_MAGENTA;
      tft.drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, ILI9341_WHITE);
    }

    if (oldcolor != currentcolor) {
      if (oldcolor == ILI9341_RED) 
        tft.fillRect(0, 0, BOXSIZE, BOXSIZE, ILI9341_RED);
      if (oldcolor == ILI9341_YELLOW) 
        tft.fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, ILI9341_YELLOW);
      if (oldcolor == ILI9341_GREEN) 
        tft.fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, ILI9341_GREEN);
      if (oldcolor == ILI9341_CYAN) 
        tft.fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, ILI9341_CYAN);
      if (oldcolor == ILI9341_BLUE) 
        tft.fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, ILI9341_BLUE);
      if (oldcolor == ILI9341_MAGENTA) 
        tft.fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, ILI9341_MAGENTA);
    }
  }
  if (((p.y-PENRADIUS) > BOXSIZE) && ((p.y+PENRADIUS) < tft.height())) {
    tft.fillCircle(p.x, p.y, PENRADIUS, currentcolor);
  }
}


